from __future__ import division
import math
import os
import re
import urllib
import sys
from collections import defaultdict

# gt is the ground truth
gt = []
res_dic = {}
dirs = []

def ndcg(alg):
  ndcg_num = []
  ndcg_den = []
  ndcg_num.append(alg[0])
  ndcg_den.append(gt[0])
  for i in range(1,len(gt)):
    try:
      ndcg_num.append(ndcg_num[i-1] + alg[i] / math.log(i+1,2))
    except IndexError:
      sys.exit("PEP IndexError, News -> NEWS ?")
    ndcg_den.append(ndcg_den[i-1] + gt[i] / math.log(i+1,2))
  return map(lambda (n,d): n/d, zip(ndcg_num,ndcg_den))

def ndcg_at(alg, rank):
  r = ndcg(alg)
  if (len(r) < rank):
    return r[-1]
  else:
    return r[rank-1]

def get_rks(path):
  f = open(path,'r')
  lines = [line.strip() for line in f]
  f.close()
  lines.reverse()
  ret = []
  debug_eval_res = set(res_dic.keys())
  debug_alg_res = set()
  for line in lines:
    url = urllib.quote_plus(urllib.unquote(line.split()[0]))
    # the input given to the algorithms can contain (although it is rare) 
    # more resources than the ground-truth obtained by crowdsourcing
    # thus the test below:
    if url in res_dic: 
      ret.append(res_dic[url])
  return ret

def mk_gt(path):
  global gt, res_dic
  f = open(path,'r')
  line = f.readline()
  res = line.split()
  line = f.readline()
  try:
    rks = map(int,line.split())
  except ValueError:
    print('transformation of ground truth ranks to integer failed')
    print(path)
    sys.exit(1)
  f.close()
  gt = sorted(rks, reverse=True)
  res_dic = {}
  for (res_name,rk_value) in zip(res,rks):
    url = urllib.quote_plus(urllib.unquote(res_name))
    res_dic[url] = rk_value

def mk_dirs():
  global dirs
  dirs_temp = []
  for d in os.walk(os.environ["DATADIR"]):
    dirs_temp.append(d[0])
  regex = re.compile(r'^.*/p[0-9]*$')
  dirs_temp = [d for d in dirs_temp if regex.search(d)]
  dirs = dirs_temp
  #f_add_dir = True
  #for d in dirs_temp:
  #  for s in ['/q0/p2','/q0/p3','/q0/p4','/q10/p4','/q11/p0','/q11/p1','/q11/p2','/q11/p3','/q11/p4','/q12/p0','/q12/p1','/q12/p2','/q12/p3','/q12/p4','/q14/p2','/q15/p2','/q15/p3','/q15/p4','/q16/p4','/q2/p1','/q2/p2','/q2/p4','/q20/p0','/q21/p2','/q22/p0','/q23/p0','/q23/p1','/q23/p3','/q28/p0','/q28/p1','/q28/p3','/q28/p4','/q3/p0','/q3/p4','/q30/p3','/q4/p1','/q4/p4']:
  #    if d.find(s) != -1: 
  #      f_add_dir = False
  #      break
  #  if f_add_dir:
  #    dirs.append(d)
  #  else:
  #    f_add_dir = True


def go():
  ret = defaultdict(list)
  K = 101
  algs = ['algcomb','alghit','algsvd','algequi']
  mk_dirs()
  for k in range(1,K):
    for alg in algs:
      ndcg_at_rk_k = []
      for d in dirs:
        mk_gt(d+"/eval")
        ndcg_at_rk_k.append(ndcg_at(get_rks(d+'/'+alg),k))
      mean_ndcg = sum(ndcg_at_rk_k)/len(ndcg_at_rk_k)
      std = 0
      for x in ndcg_at_rk_k:
        std += math.pow((x - mean_ndcg),2)
      std = math.sqrt(std / len(ndcg_at_rk_k))
      ret[k].append((mean_ndcg,std))
  for k in range(1,K):
    s = '{:3d} '.format(k)
    for j in range(0,len(algs)):
      s += '%.5f' % ret[k][j][0]
      s += ' '
      #s += '%.5f' % ret[k][j][1]
      #s += ';'
    print(s)

go()
